import requests
import json


def parse_location(latlong, access_token):
    url = get_url(latlong, access_token)

    try:
        r = requests.get(url)
        jsontext = json.loads(r.text)

        venue = jsontext['response']['venues'][0]
        name = venue['name'].encode('utf-8')
        city = venue['location'].get('city', '').encode('utf-8')
        state = venue['location'].get('state', '').encode('utf-8')
        zip = venue['location'].get('postalCode', '')
        country = venue['location'].get('country', '').encode('utf-8')
        distance = venue['location'].get('distance', '')
        category = venue['categories'][0].get('name', '').encode('utf-8')
        checkinsCount = venue['stats'].get('checkinsCount', 0)
        return [name, category, checkinsCount, zip, city, state, country, distance]
    except:
        return []


def get_url(latlong, access_token):
    if ',' in access_token:
        token = access_token.split(',')
        clientid = token[0]
        clientsecret = token[1]
        return 'https://api.foursquare.com/v2/venues/search?ll=' + str(latlong) + '&query=&limit=1&client_id=' + clientid + '&client_secret=' + clientsecret +'&v=20151216'
    else:
        return 'https://api.foursquare.com/v2/venues/search?ll=' + str(latlong) + '&query=&limit=1&oauth_token=' + access_token + '&v=20151216'



import csv
import traceback
from time import sleep
from access_tokens import access_tokens

from cutils import ProgressBar, Gmail

from parser_location import parse_location


class Foursquare:

    def __init__(self, job, **kwargs):
        self._job = job
        self._access_token = int(kwargs.get('access_token', 0))
        self._extension = ''

        if 'location' in self._job.lower():
            self._extension = '.location'
        elif 'city' in self._job.lower():
            self._extension = '.city'

    def crawl(self):
        with open(self._input_fp, 'r') as i:
            inputs = [row.strip() for row in i]
        n_retry = 10
        i_retry = 0
        bar = ProgressBar(total=len(inputs))
        with open(self._output_fp, 'a') as o:
            csvwriter = csv.writer(o, delimiter=self._output_delimiter)
            for item in inputs:
                bar.move().log()
                self.log(' ' + item + '...')

                try:
                    parsed_item = self.parse(item)
                    if parsed_item:
                        out = [item]
                        out.extend(parsed_item)
                        csvwriter.writerow(out)
                        self.log('Done.\n')
                    else:
                        self.log('NotFound.\n')
                    i_retry = 0
                except:
                    i_retry += 1
                    if i_retry <= n_retry:
                        self.throw(self._input_fp + ' ' + item + '\n' + traceback.format_exc())
                        sleep(10 * i_retry)
                        continue
                    else:
                        self.throw(self._input_fp + ' ' + item + '\n' + traceback.format_exc())
                        break
                sleep(0.72)
            self.log('\n\n' + '\nDone', True)
        bar.close()

    def set_input(self, fp_in, **kwargs):
        self._input_fp = fp_in
        self._input_index = int(kwargs.get('index', 0))
        return self

    def set_output(self, fp_out, **kwargs):
        self._output_fp = fp_out
        self._output_delimiter = kwargs.get('delimiter', ',')
        return self

    def set_notifier(self, notifier_type, **kwargs):
        if 'gmail' in notifier_type.lower():
            credential_fp = kwargs.get('credential', './')
            self._notifier = Gmail() \
                .set_from_to() \
                .set_credentials(credential_fp) \
                .set_subject('AWS Foursquare' + self._extension) \
                .send_message(self._input_fp + '\n\n\n')
            self._notifier_error = Gmail() \
                .set_from_to() \
                .set_credentials(credential_fp) \
                .set_subject('AWS Foursquare' + self._extension + ' Error!') \
                .send_message(self._input_fp + '\n\n\n')
        return self

    def log(self, message, is_close=False):
        if isinstance(self._notifier, Gmail):
            self._notifier.send_message(message)
        if is_close:
            self._notifier.close()
            self._notifier.send_message(self._input_fp + '\n\n\n')

    def throw(self, message):
        if isinstance(self._notifier_error, Gmail):
            self._notifier_error.send_message(message).close()
            self._notifier_error.send_message(self._input_fp + '\n\n\n')

    def record(self):
        self.log('KeyboardInterrupt!\n', True)

    def parse(self, item):
        if self._extension == '.location':
            return parse_location(item, access_tokens[self._access_token])



access_tokens = ['MJBT1M2HM0ADCKK5MRQQOZOYBUMLSUTUBZ4S5TJ4F5SDP4MD',
                 'UTD22EWNQ0BRGWXB21350AKPYVKD55QSGTLHJ2QROTWJ4W4J',
                 'PSKFHUEZDQA131CJBFD5LQNNWTL35G2D4YTILXRIVH5BZJOP',
                 'RUBZJ00DZZYMHTSJV0QX05GYAPNGF4ACGUFWKALDZ2QZNFVJ,RRJZVFYOAXAR54MDLSIA01E0MGUFXXVFR3FUMV03WRCYTUV4']

# client_ids = []
#
#
# client_secrets = ['RRJZVFYOAXAR54MDLSIA01E0MGUFXXVFR3FUMV03WRCYTUV4']


import csv
import datetime
import socket
import sys
import time
import traceback

import httplib2
from instagram import InstagramAPIError
from instagram import InstagramClientError
from instagram.client import InstagramAPI

from cutils.gmail import Gmail
from cutils.progress_bar import ProgressBar
from parser_media import parse_media
from parser_user_info import parse_user_info
from pyAPIs.instagram.access_tokens import access_tokens


class Instagram:

    def __init__(self, job):
        self._job = job
        self._account = 1
        self._extention = ''
        self._output = ''

        if 'follower' in job.lower():
            self._extention = '.followers'
        elif 'following' in job.lower() or 'friend' in job.lower():
            self._extention = '.following'
        elif 'photo' in job.lower() or 'media' in job.lower():
            self._extention = '.media'
        elif 'user' in job.lower():
            self._extention = '.userinfo'

        self._gmail = Gmail().set_from_to()

    def request(self, user_id, generator):
        c = 0
        n_retries_client = 0
        n_retries_socket = 0
        is_continue = True

        while True:
            if n_retries_client >= 10 or n_retries_socket >= 10:
                break
            try:
                items, _next = generator.next()

                c += 1
                sys.stdout.write(' User' + user_id + ': ' + str(c) + '  \r')
                sys.stdout.flush()

                with open(user_id + self._extention, 'a') if self._output == '' else open(self._output, 'a') as o:
                    csvwriter = csv.writer(o, delimiter=',', quotechar='"')
                    if self._extention == '.userinfo':
                        csvwriter.writerow(parse_user_info(items))
                        time.sleep(0.72)
                        break
                    for item in items:
                        if self._extention in ['.followers', '.following']:
                            csvwriter.writerow([user_id, item.id])
                        elif self._extention == '.media':
                            csvwriter.writerow(parse_media(item))

                if _next:
                    time.sleep(0.72)
                else:
                    break
            except InstagramAPIError, e:
                if str(e.status_code) == '429':
                    time.sleep(60)
                    self._gmail = self._gmail.send_message('429ExceedsLimit...')
                    self._gmail = self._gmail.send_message(datetime.datetime.now().__str__() + '\n')
                    continue
                elif str(e.status_code) == '400':
                    self._gmail = self._gmail.send_message('400NotFound...')
                    break
            except httplib2.ServerNotFoundError:
                self._gmail = self._gmail.send_message('ServerNotFoundError...')
                time.sleep(10)
                continue
            except InstagramClientError, e:
                n_retries_client += 1
                if str(e.error_message) == '402':
                    try:
                        Gmail(0).set_from_to().set_subject('AWS Instagram' + self._extention + ' Client402Error!')\
                            .send_message(user_id + '@token' + str(self._account) + ' ' + str(c) +
                                          '\n\n' + traceback.format_exc() + '\n\n')
                    except:
                        pass
                time.sleep(5)
                continue
            except socket.error:
                n_retries_socket += 1
                time.sleep(5)
                continue
            except KeyboardInterrupt:
                is_continue = False
                Gmail().set_from_to().set_subject('AWS Instagram' + self._extention + ' is being stopped!')\
                    .send_message(user_id + '@access_token ' + str(self._account)).close()
                continue
            except StopIteration:
                return is_continue
            except Exception:
                Gmail().set_from_to().set_subject('AWS Instagram' + self._extention + ' Error!')\
                    .send_message(user_id + '@server' + str(self._account) +
                                  '\nRequestError on page' + str(c) +
                                  '\n\n' + traceback.format_exc() + '\n\n').close()
                continue
        return is_continue

    def loop(self, fp):
        i = open(fp, 'r')
        n = 0
        for _ in i:
            n += 1
        i.close()
        bar = ProgressBar(total=n)

        print 'Starting', '...'
        with open(fp, 'r') as i:
            lines = [line.strip() for line in i]
        is_continue = True
        for user_id in lines:
            if is_continue:
                bar = bar.move().log()
                self._gmail = self._gmail.send_message('User' + user_id)
                generator = {'.followers': self._api.user_followed_by(user_id=user_id, as_generator=True, max_pages=999999),
                             '.following': self._api.user_follows(user_id=user_id, as_generator=True, max_pages=999999),
                             '.media': self._api.user_recent_media(user_id=user_id, as_generator=True, max_pages=60),
                             '.userinfo': self._api.user(user_id=user_id, as_generator=True)
                             }.get(self._extention)
                is_continue = self.request(user_id, generator)
                self._gmail = self._gmail.send_message('Done!\n')
            else:
                self._gmail = self._gmail.close()
                break
        if is_continue:
            self._gmail = self._gmail.send_message('\n' + fp + ' Done.\n').close()
        bar.close()

        return True

    def test(self, user_id):
        self.request(user_id,
                     {'.followers': self._api.user_followed_by(user_id=user_id, as_generator=True, max_pages=999999),
                      '.following': self._api.user_follows(user_id=user_id, as_generator=True, max_pages=999999),
                      '.media': self._api.user_recent_media(user_id=user_id, as_generator=True, max_pages=60)
                      }.get(self._extention))

    def set_access_token(self, i):
        self._account = i
        self._api = InstagramAPI(access_token=access_tokens[self._account])
        self._gmail = self._gmail.set_subject('AWS Instagram' + self._extention + '_' + str(i))
        return self

    def set_output(self, output):
        self._output = output
        return self

# if __name__ == '__main__':
#     argvs = sys.argv[1:]
#     job_name = argvs[0]
#     filepath = argvs[1]
#     server = int(argvs[2])
#     ins = Instagram(job_name).set_access_token(server).loop(filepath)


# Instagram('followers').set_access_token(19).test('294211877')